@@ -88,6 +88,7 @@
 #define TRUST_MOK          (1 << 27)
 #define UNTRUST_MOK        (1 << 28)
 #define SET_SBAT           (1 << 29)
+#define SET_SSP            (1 << 30)
 
 #define DEFAULT_CRYPT_METHOD SHA512_BASED
 #define DEFAULT_SALT_SIZE    SHA512_SALT_MAX
@@ -132,17 +133,22 @@ print_help ()
 	printf ("  --import-hash <hash>\t\t\tImport a hash into MOK or MOKX\n");
 	printf ("  --delete-hash <hash>\t\t\tDelete a hash in MOK or MOKX\n");
 	printf ("  --set-verbosity <true/false>\t\tSet the verbosity bit for shim\n");
-	printf ("  --set-fallback-verbosity <true/false>\t\tSet the verbosity bit for fallback\n");
-	printf ("  --set-fallback-noreboot <true/false>\t\tPrevent fallback from automatically rebooting\n");
+	printf ("  --set-fallback-verbosity <true/false>"
+			"\tSet the verbosity bit for fallback\n");
+	printf ("  --set-fallback-noreboot <true/false>"
+			"\tPrevent fallback from automatically rebooting\n");
 	printf ("  --trust-mok\t\t\t\tTrust MOK keys within the kernel keyring\n");
 	printf ("  --untrust-mok\t\t\t\tDo not trust MOK keys\n");
-	printf ("  --set-sbat-policy <latest/previous/delete>\t\tApply Latest, Previous, or Blank SBAT revocations\n");
+	printf ("  --set-sbat-policy <latest/automatic>"
+			"\tApply Latest or Automatic SBAT revocations\n");
+	printf ("  --set-ssp-policy <latest/automatic/delete>\n"
+			"\t\t\t\t\tApply Latest, Automatic, or delete SkuSiPolicy\n");
 	printf ("  --pk\t\t\t\t\tList the keys in PK\n");
 	printf ("  --kek\t\t\t\t\tList the keys in KEK\n");
 	printf ("  --db\t\t\t\t\tList the keys in db\n");
 	printf ("  --dbx\t\t\t\t\tList the keys in dbx\n");
 	printf ("  --timeout <-1,0..0x7fff>\t\tSet the timeout for MOK prompt\n");
-	printf ("  --list-sbat-revocations\t\t\t\tList the entries in SBAT\n");
+	printf ("  --list-sbat-revocations\t\tList the entries in SBAT\n");
 	printf ("\n");
 	printf ("Supplimentary Options:\n");
 	printf ("  --hash-file <hash file>\t\tUse the specific password hash\n");
@@ -164,7 +170,16 @@ list_keys (const uint8_t *data, const size_t data_size)
 	}
 
 	for (unsigned int i = 0; i < mok_num; i++) {
+		char *owner_str = NULL;
+		int ret;
 		printf ("[key %d]\n", i+1);
+
+		ret = efi_guid_to_str(&list[i].owner, &owner_str);
+		if (ret > 0) {
+			printf ("Owner: %s\n", owner_str);
+			free (owner_str);
+		}
+
 		efi_guid_t sigtype = list[i].header->SignatureType;
 		if (efi_guid_cmp (&sigtype, &efi_guid_x509_cert) == 0) {
 			print_x509 (list[i].mok, list[i].mok_size);
@@ -194,7 +209,13 @@ list_keys_in_var (const char *var_name, const efi_guid_t guid)
 	if (ret >= 0) {
 		ret = list_keys (data, data_sz);
 		free(data);
-		return ret;
+		if (ret != -1)
+			return 0;
+		/*
+		 * If list_keys() returns -1, then we have problem
+		 * with the data from /sys/firmware/efi/mok-variables/.
+		 * Continue to try our luck with efivar/efivarfs.
+		 */
 	}
 
 	for (i = 0; i < SIZE_MAX; i++) {
@@ -524,9 +545,9 @@ error:
 }
 
 static int
-is_one_duplicate (const efi_guid_t *type,
-		  const void *data, const uint32_t data_size,
-		  uint8_t *var_data, size_t var_data_size)
+check_one_duplicate (const efi_guid_t *type,
+		     const void *data, const uint32_t data_size,
+		     uint8_t *var_data, size_t var_data_size)
 {
 	uint32_t node_num;
 	MokListNode *list;
@@ -536,9 +557,8 @@ is_one_duplicate (const efi_guid_t *type,
 		return 0;
 
 	list = build_mok_list (var_data, var_data_size, &node_num);
-	if (list == NULL) {
-		goto done;
-	}
+	if (list == NULL)
+		return -1;
 
 	for (unsigned int i = 0; i < node_num; i++) {
 		efi_guid_t sigtype = list[i].header->SignatureType;
@@ -562,9 +582,7 @@ is_one_duplicate (const efi_guid_t *type,
 		}
 	}
 
-done:
-	if (list)
-		free (list);
+	free (list);
 
 	return ret;
 }
@@ -585,12 +603,17 @@ is_duplicate (const efi_guid_t *type,
 		size_t var_data_size = 0;
 		ret = mok_get_variable(db_name, &var_data, &var_data_size);
 		if (ret >= 0) {
-			ret = is_one_duplicate(type, data, data_size,
-					       var_data, var_data_size);
+			ret = check_one_duplicate(type, data, data_size,
+						  var_data, var_data_size);
 			if (ret >= 0) {
 				free (var_data);
 				return ret;
 			}
+			/*
+			 * If check_duplicate() returns -1, then we have problem
+			 * with the data from /sys/firmware/efi/mok-variables/.
+			 * Continue to try our luck with efivar/efivarfs.
+			 */
 			var_data = NULL;
 			var_data_size = 0;
 		}
@@ -611,8 +634,8 @@ is_duplicate (const efi_guid_t *type,
 		if (ret < 0)
 			return 0;
 
-		ret = is_one_duplicate(type, data, data_size,
-				       var_data, var_data_size);
+		ret = check_one_duplicate(type, data, data_size,
+					  var_data, var_data_size);
 		free (var_data);
 		/*
 		 * If ret is < 0, the next one will error as well.
@@ -799,7 +822,7 @@ print_skip_message (const char *filename, const void *mok,
 			printf ("%s is blocked in dbx\n", filename);
 		else if (is_duplicate (&efi_guid_x509_cert, mok, mok_size,
 				       &efi_guid_shim, "MokListXRT"))
-			printf ("%s is blocked in MokListXRT\n", filename);
+			printf ("%s is blocked in MokListX\n", filename);
 		break;
 	case DELETE_MOK:
 		if (!is_duplicate (&efi_guid_x509_cert, mok, mok_size,
@@ -1216,6 +1239,7 @@ export_db_keys (const DBName db_name)
 
 	list = build_mok_list (data, data_size, &mok_num);
 	if (list == NULL) {
+		free(data);
 		return -1;
 	}
 
@@ -1756,21 +1780,27 @@ list_db (const DBName db_name)
 }
 
 static int
-manage_sbat (const uint8_t sbat_policy)
+manage_policy (unsigned int command, const uint8_t policy)
 {
-	if (sbat_policy) {
+	const char *varname;
+	if (command == SET_SBAT)
+		varname = "SbatPolicy";
+	if (command == SET_SSP)
+		varname = "SSPPolicy";
+
+	if (policy) {
 		uint32_t attributes = EFI_VARIABLE_NON_VOLATILE
 				      | EFI_VARIABLE_BOOTSERVICE_ACCESS
 				      | EFI_VARIABLE_RUNTIME_ACCESS;
-		if (efi_set_variable (efi_guid_shim, "SbatPolicy",
-				      (uint8_t *)&sbat_policy,
-				      sizeof (sbat_policy),
+		if (efi_set_variable (efi_guid_shim, varname,
+				      (uint8_t *)&policy,
+				      sizeof (policy),
 				      attributes, S_IRUSR | S_IWUSR) < 0) {
 			fprintf (stderr, "Failed to set SbatPolicy\n");
 			return -1;
 		}
 	} else {
-		return test_and_delete_mok_var ("SbatPolicy");
+		return test_and_delete_mok_var (varname);
 	}
 	return 0;
 }
@@ -1791,7 +1821,7 @@ main (int argc, char *argv[])
 	uint8_t verbosity = 0;
 	uint8_t fb_verbosity = 0;
 	uint8_t fb_noreboot = 0;
-	uint8_t sbat_policy = 0;
+	uint8_t policy = 0;
 	DBName db_name = MOK_LIST_RT;
 	int ret = -1;
 	int sb_check;
@@ -1799,11 +1829,6 @@ main (int argc, char *argv[])
 	force_ca_check = 0;
 	check_keyring = 1;
 
-	if (!efi_variables_supported ()) {
-		fprintf (stderr, "EFI variables are not supported on this system\n");
-		exit (1);
-	}
-
 	while (1) {
 		static struct option long_options[] = {
 			{"help",               no_argument,       0, 'h'},
@@ -1837,6 +1862,7 @@ main (int argc, char *argv[])
 			{"trust-mok",          no_argument,       0, 0  },
 			{"untrust-mok",        no_argument,       0, 0  },
 			{"set-sbat-policy",    required_argument, 0, 0  },
+			{"set-ssp-policy",     required_argument, 0, 0  },
 			{"pk",                 no_argument,       0, 0  },
 			{"kek",                no_argument,       0, 0  },
 			{"db",                 no_argument,       0, 0  },
@@ -1851,7 +1877,7 @@ main (int argc, char *argv[])
 		};
 
 		int option_index = 0;
-		c = getopt_long (argc, argv, "cd:f:g::hi:lmpst:xDNPXv",
+		c = getopt_long (argc, argv, "cd:f:g::hi:lmpt:xDNPXv",
 				 long_options, &option_index);
 
 		if (c == -1)
@@ -1929,11 +1955,23 @@ main (int argc, char *argv[])
 			} else if (strcmp (option, "set-sbat-policy") == 0) {
 				command |= SET_SBAT;
 				if (strcmp (optarg, "latest") == 0)
-					sbat_policy = 1;
-				else if (strcmp (optarg, "previous") == 0)
-					sbat_policy = 2;
+					policy = 1;
+				else if ((strcmp (optarg, "previous") == 0) ||
+					 (strcmp (optarg, "automatic") == 0))
+					policy = 2;
 				else if (strcmp (optarg, "delete") == 0)
-					sbat_policy = 3;
+					policy = 3;
+				else
+					command |= HELP;
+			} else if (strcmp (option, "set-ssp-policy") == 0) {
+				command |= SET_SSP;
+				if (strcmp (optarg, "latest") == 0)
+					policy = 1;
+				else if ((strcmp (optarg, "previous") == 0) ||
+					 (strcmp (optarg, "automatic") == 0))
+					policy = 2;
+				else if (strcmp (optarg, "delete") == 0)
+					policy = 3;
 				else
 					command |= HELP;
 			} else if (strcmp (option, "pk") == 0) {
@@ -2015,6 +2053,10 @@ main (int argc, char *argv[])
 			for (i = 0; i < total; i++) {
 				f_ind = i + optind - 1;
 				files[i] = malloc (strlen(argv[f_ind]) + 1);
+				if (files[i] == NULL) {
+					fprintf (stderr, "Could not allocate space: %m\n");
+					exit(1);
+				}
 				strcpy (files[i], argv[f_ind]);
 			}
 
@@ -2084,19 +2126,24 @@ main (int argc, char *argv[])
 			break;
 		case 'v':
 			printf ("%s\n", VERSION);
+			ret = 0;
 			goto out;
 		case 'h':
 		case '?':
+		default:
 			command |= HELP;
 			break;
-		default:
-			abort ();
 		}
 	}
 
 	if (pw_hash_file && use_root_pw)
 		command |= HELP;
 
+	if (!(command & HELP) && !efi_variables_supported ()) {
+		fprintf (stderr, "EFI variables are not supported on this system\n");
+		exit (1);
+	}
+
 	if (db_name != MOK_LIST_RT && !(command & ~MOKX))
 		command |= LIST_ENROLLED;
 
@@ -2243,10 +2290,12 @@ main (int argc, char *argv[])
 			ret = print_var_content ("SbatLevelRT", efi_guid_shim);
 			break;
 		case SET_SBAT:
-			ret = manage_sbat(sbat_policy);
+		case SET_SSP:
+			ret = manage_policy(command, policy);
 			break;
 		default:
 			print_help ();
+			ret = 0;
 			break;
 	}
 
